/*
  This file is part of GerPhys.

  GerPhys is free software: you can redistribute it and/or modify it under the
  terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerPhys is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerPhys.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "vector.h"

#include <math.h>
#include <stdexcept>


namespace GerPh {

  Vector::Vector(const Vector &aTemplate) {
    fDimension = aTemplate.getDimension();
    fCoeff = new double[fDimension];
    for (int i = 0; i < fDimension; i++) {
      fCoeff[i] = aTemplate.getAt(i);
    } // for
  } // Vector::Vector

  Vector::Vector(int aDimension, double *aCoeff) {
    fDimension = aDimension;
    fCoeff = new double[fDimension];
    for (int i = 0; i < fDimension; i++) {
      if (aCoeff) {
	fCoeff[i] = aCoeff[i];
      } else {
        fCoeff[i] = 0.0;
      } // if
    } // for
  } // Vector::Vector

  Vector::~Vector() {
    free(fCoeff);
  } // Vector::~Vector

  Vector& Vector::addVec(const Vector &v) {
    checkDimension(v, "Vector += ");
    for (int i = 0; i < fDimension; i++) {
      fCoeff[i] += v.getAt(i);
    } // for
    return *this;
  } // Vector::addVec

  int Vector::checkDimension(const Vector &v, const QString &aCall) const {
    int dim = v.getDimension();
    if (dim != fDimension) {
      throw std::invalid_argument(qPrintable(QString().sprintf(
        "%s: vector dimension conflict (%i != %i)",
	qPrintable(aCall), dim, fDimension)));
    } // if
    return fDimension;
  } // Vector::checkDimension

  double Vector::getAbs() {
    double sum = 0.0;
    for (int i = 0; i < fDimension; i++) {
      double x = fCoeff[i];
      sum += x * x;
    } // for
    return sqrt(sum);
  } // Vector::getAbs

  double Vector::getAt(int i) const {
    return fCoeff[i];
  } // Vector::getAt

  int Vector::getDimension() const {
    return fDimension;
  } // Vector::getDimension

  bool Vector::isEqual(const Vector &v) const {
    if (v.getDimension() != fDimension) {
      return false;
    } // if
    for (int i = 0; i < fDimension; i++) {
      if (fCoeff[i] != v.getAt(i)) {
        return false;
      } // if
    } // for
    return true;
  } // Vector::isEqual

  Vector& Vector::multDbl(double x) {
    for (int i = 0; i < fDimension; i++) {
      fCoeff[i] *= x;
    } // for
    return *this;
  } // Vector::multDbl

  Vector* Vector::setAt(int i, double aValue) {
    fCoeff[i] = aValue;
    return this;
  } // Vector::setAt

  Vector& Vector::setEqual(const Vector &v) {
    checkDimension(v, "Vector::setEqual()");
    for (int i = 0; i < fDimension; i++) {
      fCoeff[i] = v.getAt(i);
    } // for
    return *this;
  } // Vector::setEqual

  Vector& Vector::subVec(const Vector &v) {
    checkDimension(v, "Vector -= ");
    for (int i = 0; i < fDimension; i++) {
      fCoeff[i] -= v.getAt(i);
    } // for
    return *this;
  } // Vector::subVec

  Vector operator+ (const Vector &v1, const Vector &v2) {
    Vector result = v1;
    result += v2;
    return result;
  } // operator+

  Vector operator- (const Vector &v1, const Vector &v2) {
    Vector result = v1;
    result -= v2;
    return result;
  } // operator-

  double operator* (const Vector &v1, const Vector &v2) {
    int n = v1.checkDimension(v2, "Vector * Vector");
    double result = 0.0;
    for (int i = 0; i < n; i++) {
      result += v1.getAt(i) * v2.getAt(i);
    } // for
    return result;
  } // operator*

  Vector operator* (double x, const Vector &v) {
    Vector result = v;
    result *= x;
    return result;
  } // operator*

  Vector operator* (const Vector &v, double x) {
    Vector result = v;
    result *= x;
    return result;
  } // operator*

  Vector operator/ (const Vector &v, double x) {
    Vector result = v;
    result /= x;
    return result;
  } // operator/

} // GerPh
